import arrow


def format_time(timestamp: int) -> str:
    """
    Converts Unix timestamp to a human readable time
    in the format h:mm:ss AM/PM
               or h:mm AM/PM if the seconds are '00'
               or h AM/PM if the minutes are '00'
    """
    # Current time is in EET zone, so convert it to UTC
    current_time = arrow.Arrow.fromtimestamp(timestamp).to('UTC')
    formatted_time = current_time.format('h:mm:ss A')
    if current_time.format('ss') == '00':
        formatted_time = current_time.format('h:mm A')
        if current_time.format('mm') == '00':
            formatted_time = current_time.format('h A')

    return formatted_time


def format_time_range(opening_time: str, closing_time: str) -> str:
    """ Returns a formatted string representing an opening time and closing time pair. """
    return f'{opening_time} - {closing_time}'


def format_output(data: dict) -> str:
    """ Returns the human readable string from the formatted dict. """
    lines = []
    for day, times in data.items():
        if times:
            lines.append(f'{day.title()}: {", ".join(times)}')
        else:
            lines.append(f'{day.title()}: Closed')
    formatted_output = '\n'.join(lines)

    return formatted_output



result_schema = {'monday': [], 'tuesday': [], 'wednesday': [], 'thursday': [], 'friday': [], 'saturday': [],
                 'sunday': []}

main_json_schema_structure = {
    "type": "object",
    "properties": {
        "monday": {
            "type": "array",
            "items": {}
        },
        "tuesday": {
            "type": "array",
            "items": {}
        },
        "wednesday": {
            "type": "array",
            "items": {}
        },
        "thursday": {
            "type": "array",
            "items": {}
        },
        "friday": {
            "type": "array",
            "items": {}
        },
        "saturday": {
            "type": "array",
            "items": {}
        },
        "sunday": {
            "type": "array",
            "items": {}
        }
    },
    "required": [
        "monday",
        "tuesday",
        "wednesday",
        "thursday",
        "friday",
        "saturday",
        "sunday"
    ]
}



from calc_work_hours.controller import CalculateWorkHours
from calc_work_hours.json_utils import read_json_file, validate_json_file

""" function which user needs to use to get the working hours intervals in human readable format"""


def calculate_working_hours(json_file_path):
    data = read_json_file(file_path=json_file_path)
    isValidated = validate_json_file(Json_file_object=data)

    if isValidated:
        calc_hours_work = CalculateWorkHours(json_data=data)
        calc_hours_work.refactor_input_json_with_correct_data()
        formatted_output = calc_hours_work.working_hours_in_human_readable_format()

        return formatted_output



from setuptools import setup, find_packages

setup(
    name='workhourcalc',
    version='0.0.1',
    description="Calculate working hours",
    long_description='Gets the input opening and closing time of restaurants in millisecond and writes the working '
                     'hours intervals in human readable format',
    url='https://github.com/rasindulakkitha/calc_work_hours',
    author='Rasindu Wanigasinghe',
    author_email='rasindulakkitha95@gmail.com',
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Operating System :: OS Independent",
    ],
    keywords='work_hour',
    packages=find_packages(exclude=('tests*', 'testing*')),
    install_requires=['arrow','jsonschema']
)



import sys

from calc_work_hours import DAYS
from calc_work_hours.schema import result_schema
from calc_work_hours.utils import format_time, format_time_range, format_output

"""" implement calculate work hours in here. first refactor the code and setup the json with correct events.
    then implement the working hours interval with human readable format"""


class CalculateWorkHours:
    """ create the init method for the class
    """

    def __init__(self, json_data):
        self.refactored_schema = None
        self.data = json_data

    """
    refactor the json object according to the below rules.
    1. if sequence of opening/closing time are not matched(for example, two sequence closing events) 
    then raise an exception
    2. sort event values
    3.if opening time is in the specified day(let's say Monday) and the related closing event in the next day(Tuesday) 
    then the closing event must be moved to the previous day(Monday)"""

    def refactor_input_json_with_correct_data(self):
        try:

            validated_json_data = self.data

            """ iterate with DAYS and refactor the json object"""
            for day in DAYS:
                if len(validated_json_data[day]) == 0:
                    result_schema[day] = []
                else:

                    previous_type = None

                    """ sort the validated json object"""
                    unsorted_list = validated_json_data[day]
                    sorted_events_list = sorted(unsorted_list, key=lambda i: i['value'])

                    for event in sorted_events_list:
                        if sorted_events_list.index(event) == 0:
                            if event['type'] == 'open':
                                previous_type = event['type']
                                result_schema[day].append(event)
                            else:
                                """ sort the validated json object"""
                                previous_day_data = validated_json_data[DAYS[DAYS.index(day) - 1]]
                                previous_day_data = sorted(previous_day_data, key=lambda i: i['value'])
                                previous_day_last_event = previous_day_data[- 1]

                                if previous_day_last_event["type"] == event['type']:
                                    raise Exception(f'Cannot have two consecutive "{event["type"]}" times.')
                                else:
                                    previous_type = event['type']
                                    result_schema[DAYS[DAYS.index(day) - 1]].append(event)
                        else:
                            if previous_type == event['type']:
                                raise Exception(f'Cannot have two consecutive "{event["type"]}" times.')
                            else:
                                previous_type = event['type']
                                result_schema[day].append(event)

            self.refactored_schema = result_schema

            return result_schema
        except Exception as error:
            print(sys.exc_info())
            raise Exception(error)

    """
        format the refactored data in human readable format using below util functions
        format_time,format_time_range,format_output
        """

    def working_hours_in_human_readable_format(self):
        try:
            refactored_data = self.refactored_schema

            work_hours_dict = dict()

            for day, events in refactored_data.items():
                opening_time = None
                formatted_times_list = []
                for event in events:

                    if event['type'] == 'open':
                        opening_time = format_time(event['value'])
                    else:
                        closing_time = format_time(event['value'])

                        formatted_time = format_time_range(opening_time, closing_time)
                        formatted_times_list.append(formatted_time)

                work_hours_dict[day] = formatted_times_list

            formatted_output = format_output(work_hours_dict)

            return formatted_output

        except Exception as error:
            print(sys.exc_info())
            raise Exception(error)



import json
import sys

from jsonschema import validate
import jsonschema

from calc_work_hours import TYPES, EVENT_CLOSE, EVENT_OPEN, DAYS
from calc_work_hours.schema import main_json_schema_structure

""" read the json file from the file path. and also this validates the json file. if its not in json format then
    it will be raise an exception"""


def read_json_file(file_path):
    try:
        with open(file_path) as json_file:
            data = json.load(json_file)
            """ sort the json values from time"""
            for key, value in data.items():
                data[key] = sorted(value, key=lambda i: i['value'])

            return data
    except ValueError as err:
        raise Exception(err)


""" validate the extracted json object in here.
    1. validate the json object using predefined schema
    2. validate the json object with the custom validation according to the requirement"""


def validate_json_file(Json_file_object):
    """validate the json structure using predefined schema"""
    try:
        validate(instance=Json_file_object, schema=main_json_schema_structure)
    except jsonschema.exceptions.ValidationError as err:
        raise Exception("Invalid json format found in json file - " + str(err))

    """validate the nested json data with custom validation rules"""
    try:
        """validated json data with below scenarios.
            1. week cannot start with close event
            2. week cannot end with open event
            3. start and end of the events cannot be same"""

        input_json_nested_data_list = [value for key, value in Json_file_object.items() if len(value) > 0]

        for event_items in input_json_nested_data_list:
            event_items = sorted(event_items, key=lambda i: i['value'])

            err_data_list = [event['type'] for event in event_items if event['type'] not in TYPES]
            if len(err_data_list) > 0:
                raise Exception("Invalid event type found in json file.")

        start_day_list = [event['type'] for event in input_json_nested_data_list[0] if event['type'] in TYPES]
        week_start_event = str(start_day_list[0])
        end_day_list = [event['type'] for event in input_json_nested_data_list[-1] if event['type'] in TYPES]
        week_end_event = str(end_day_list[-1])

        if week_start_event == EVENT_CLOSE or week_end_event == EVENT_OPEN:
            raise Exception("Invalid event type found in starting/ending of the week in json file.")
        if week_start_event == week_end_event:
            raise Exception("Start and end a week with events of distinct types. check the json file")

    except Exception as err:
        print(sys.exc_info())
        raise Exception(err)

    return True
